@@ -80,6 +80,8 @@ Add_dialog_data *xa_create_add_dialog()
 	gtk_window_set_position (GTK_WINDOW (add_dialog->dialog1),GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_type_hint (GTK_WINDOW (add_dialog->dialog1), GDK_WINDOW_TYPE_HINT_DIALOG);
 
+	xa_set_xarchiver_icon(GTK_WINDOW(add_dialog->dialog1));
+
 	dialog_vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(add_dialog->dialog1));
 
 	add_dialog->notebook1 = gtk_notebook_new ();
@@ -234,11 +236,14 @@ Add_dialog_data *xa_create_add_dialog()
 void xa_set_add_dialog_options(Add_dialog_data *add_dialog,XArchive *archive)
 {
 	GTK_COMPAT_TOOLTIPS;
-	gboolean flag = FALSE;
+	gboolean epub, flag = FALSE;
 	gchar *compression_msg = NULL;
 	gushort default_value, max_value;
 	default_value = max_value = 0;
 
+	if (progress)
+		gtk_widget_hide(progress->window);
+
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->check_save_geometry)) && prefs_window->add_coords[0] != -1)
 		gtk_window_set_default_size (GTK_WINDOW(add_dialog->dialog1), prefs_window->add_coords[0], prefs_window->add_coords[1]);
 	else
@@ -246,9 +251,11 @@ void xa_set_add_dialog_options(Add_dialog_data *add_dialog,XArchive *archive)
 
 	prefs_window->size_changed[1] = TRUE;
 
+	epub = (archive->tag == 'e');
+
 	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(add_dialog->filechooserwidget1), !SINGLE_FILE_COMPRESSOR(archive));
 
-	if ((archive->location_path && *archive->location_path) || !archive->can_full_path[1])
+	if ((archive->location_path && *archive->location_path) || !archive->can_full_path[1] || epub)
 	{
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(add_dialog->no_store_path), TRUE);
 		flag = FALSE;
@@ -331,7 +338,8 @@ void xa_set_add_dialog_options(Add_dialog_data *add_dialog,XArchive *archive)
 		g_signal_connect(G_OBJECT(compression_value), "value-changed", G_CALLBACK(fix_adjustment_value), NULL);
 	gtk_widget_set_tooltip_text(add_dialog->compression_scale, compression_msg);
 
-	gtk_widget_set_sensitive(add_dialog->add_password, archive->can_password);
+	gtk_widget_set_sensitive(add_dialog->add_password, archive->can_password && !epub);
+	gtk_widget_set_sensitive(add_dialog->add_password_entry, archive->can_password && !epub);
 	gtk_widget_show_all(add_dialog->dialog1);
 }
 
@@ -358,6 +366,11 @@ void xa_parse_add_dialog_options (XArchive *archive,Add_dialog_data *add_dialog)
 				xa_show_message_dialog (GTK_WINDOW (xa_main_window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Can't add files to the archive:"), _("You haven't selected any files to add!") );
 				break;
 			}
+			if (SINGLE_FILE_COMPRESSOR(archive) && g_file_test(g_slist_nth_data(list, 0), G_FILE_TEST_IS_DIR))
+			{
+				xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("An error occurred!"), _("The archiver doesn't support compression of directories or multiple files!"));
+				break;
+			}
 			if (gtk_widget_is_sensitive(add_dialog->add_password) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->add_password)))
 			{
 				temp_password  = g_strdup (gtk_entry_get_text ( GTK_ENTRY (add_dialog->add_password_entry) ));
@@ -462,12 +475,10 @@ void xa_execute_add_commands (XArchive *archive, GSList *list, gboolean recurse,
 		}
 		xa_set_button_state(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0);
 	}
-	else
-		xa_show_progress_bar(archive);
 
 	while (list)
 	{
-		xa_recurse_local_directory((gchar*) list->data, &dirlist, recurse);
+		xa_recurse_local_directory((gchar*) list->data, &dirlist, archive->do_full_path, recurse);
 		list = list->next;
 	}
 	files = xa_collect_filenames(archive, dirlist);
