@@ -77,7 +77,7 @@ static gboolean xa_process_stdout (GIOChannel *ioc, GIOCondition cond, XArchive
 	g_io_channel_shutdown(ioc, FALSE, NULL);
 	g_io_channel_unref(ioc);
 
-	xa_child_processed(XA_CHILD_STDOUT, cond == G_IO_HUP, archive);
+	xa_child_processed(XA_CHILD_STDOUT, cond != G_IO_HUP, archive);
 
 	return FALSE;
 }
@@ -122,22 +122,25 @@ static gboolean xa_process_stderr (GIOChannel *ioc, GIOCondition cond, XArchive
 	g_io_channel_shutdown(ioc, FALSE, NULL);
 	g_io_channel_unref(ioc);
 
-	xa_child_processed(XA_CHILD_STDERR, cond == G_IO_HUP, archive);
+	xa_child_processed(XA_CHILD_STDERR, cond != G_IO_HUP, archive);
 
 	return FALSE;
 }
 
 static void xa_process_exit (GPid pid, gint status, XArchive *archive)
 {
-	gboolean result;
-
 	if (WIFEXITED(status))
 	{
-		result = (WEXITSTATUS(status) == 0 || (archive->status == XARCHIVESTATUS_RELOAD &&
-		                                       !g_file_test(archive->path[0], G_FILE_TEST_EXISTS)));
-		g_spawn_close_pid(pid);
-		xa_child_processed(XA_CHILD_EXIT, result, archive);
+		if ((archive->status == XARCHIVESTATUS_RELOAD) && !g_file_test(archive->path[0], G_FILE_TEST_EXISTS))
+			status = 0;
+		else
+			status = WEXITSTATUS(status);
 	}
+	else
+		status = 1;
+
+	g_spawn_close_pid(pid);
+	xa_child_processed(XA_CHILD_EXIT, status, archive);
 }
 
 static void xa_delete_working_directory (XArchive *archive)
@@ -411,10 +414,17 @@ void xa_spawn_async_process (XArchive *archive, const gchar *command)
 	if (xa_main_window)
 	{
 		gtk_widget_set_sensitive(Stop_button, TRUE);
-		g_timeout_add(350, (GSourceFunc) xa_flash_led_indicator, archive);
+
+		if (!archive->timeout)
+			archive->timeout = g_timeout_add(350, (GSourceFunc) xa_flash_led_indicator, archive);
+	}
+	else if (!progress || !progress->multi_extract)
+	{
+		xa_show_progress_bar(archive);
+
+		if (!archive->timeout)
+			archive->timeout = g_timeout_add(100, (GSourceFunc) xa_pulse_progress_bar, archive);
 	}
-	else if (progress && !progress->multi_extract)
-		g_timeout_add(100, xa_pulse_progress_bar, NULL);
 
 	if (archive->output)
 	{
@@ -484,6 +494,7 @@ void xa_clean_archive_structure (XArchive *archive)
 	g_free(archive->path[0]);
 	g_free(archive->path[1]);
 	g_free(archive->path[2]);
+	g_free(archive->path[3]);
 	g_free(archive->destination_path);
 	g_free(archive->extraction_dir);
 	g_free(archive->password);
@@ -574,16 +585,11 @@ gboolean xa_run_command (XArchive *archive, const gchar *command)
 {
 	pid_t pid = 0;
 	int status;
-	gboolean result;
-
-	/* batch mode and archive's list function has completed */
-	if (!xa_main_window && archive->column_types)
-		xa_show_progress_bar(archive);
 
 	xa_spawn_async_process(archive, command);
 
 	if (archive->child_pid == 0)
-		result = FALSE;
+		status = 1;
 	else
 	{
 		while (pid != archive->child_pid && pid != -1)
@@ -594,12 +600,15 @@ gboolean xa_run_command (XArchive *archive, const gchar *command)
 				gtk_main_iteration();
 		}
 
-		result = (WIFEXITED(status) && (WEXITSTATUS(status) == 0));
+		if (WIFEXITED(status))
+			status = WEXITSTATUS(status);
+		else
+			status = 1;
 	}
 
-	xa_child_processed(XA_CHILD_EXIT, result, archive);
+	xa_child_processed(XA_CHILD_EXIT, status, archive);
 
-	return result;
+	return (status == 0 || (guint8) status == archive->exitstatus_ok);
 }
 
 gboolean xa_has_containing_directory (XArchive *archive)
