@@ -150,7 +150,7 @@ struct _XArchive
 	GSList *forward;
 	XAClipboard *clipboard;
 	/* environment */
-	gchar *path[3];          // 0: original, 1: escaped, 2: working copy
+	gchar *path[4];          // 0: original, 1: escaped, 2: working copy, 3: escaped working copy
 	gchar *working_dir;
 	gchar *destination_path;
 	gchar *extraction_dir;   // (free before use)
@@ -187,6 +187,7 @@ struct _XArchive
 	gboolean do_move;
 	gboolean do_solid;
 	/* child process */
+	guint timeout;
 	gchar *child_dir;     // (free and set NULL after use)
 	GPid child_pid;
 	guint child_ref;
@@ -197,6 +198,7 @@ struct _XArchive
 	/* (un)compressor interface */
 	XArchiver *archiver;
 	parse_output_func parse_output;
+	guint8 exitstatus_ok;
 };
 
 extern XArchive *archive[];
