@@ -54,6 +54,8 @@ static void xa_select_where_to_extract (GtkEntry *entry, gint icon_pos, GTK_COMP
 							GTK_RESPONSE_ACCEPT,
 							NULL);
 
+	xa_set_xarchiver_icon(GTK_WINDOW(file_selector));
+
 	current_path = gtk_entry_get_text(GTK_ENTRY(entry));
 	if (strlen(current_path) > 0)
 		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER (file_selector),current_path);
@@ -301,6 +303,8 @@ Extract_dialog_data *xa_create_extract_dialog()
 	gtk_window_set_type_hint (GTK_WINDOW (dialog_data->dialog1),GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_window_set_destroy_with_parent(GTK_WINDOW (dialog_data->dialog1),TRUE);
 
+	xa_set_xarchiver_icon(GTK_WINDOW(dialog_data->dialog1));
+
 	dialog_data->dialog_vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dialog_data->dialog1));
 
 	vbox1 = gtk_vbox_new (FALSE,2);
@@ -434,6 +438,9 @@ Extract_dialog_data *xa_create_extract_dialog()
 
 void xa_set_extract_dialog_options(Extract_dialog_data *dialog_data,gint selected,XArchive *archive)
 {
+	if (progress)
+		gtk_widget_hide(progress->window);
+
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->check_save_geometry))&& prefs_window->extract_dialog[0] != -1)
 		gtk_window_set_default_size (GTK_WINDOW(dialog_data->dialog1),prefs_window->extract_dialog[0],prefs_window->extract_dialog[1]);
 	else
@@ -642,6 +649,8 @@ Multi_extract_data *xa_create_multi_extract_dialog()
 	gtk_widget_set_size_request(dialog_data->multi_extract,-1,300);
 	gtk_window_set_title (GTK_WINDOW (dialog_data->multi_extract),_("Multi-Extract"));
 
+	xa_set_xarchiver_icon(GTK_WINDOW(dialog_data->multi_extract));
+
 	dialog_vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dialog_data->multi_extract));
 	vbox1 = gtk_vbox_new (FALSE,5);
 	gtk_box_pack_start (GTK_BOX (dialog_vbox1),vbox1,TRUE,TRUE,0);
@@ -839,7 +848,6 @@ run:
 
 	xa_show_progress_bar(NULL);
 
-	percent=fraction;
 	do
 	{
 		gtk_tree_model_get(GTK_TREE_MODEL(dialog->files_liststore), &iter, 0, &file, 2, &path, -1);
@@ -860,9 +868,15 @@ run:
 		if (dialog->stop_pressed)
 			break;
 		percent += fraction;
+
+		xa_increase_progress_bar(progress, NULL, percent);
 	}
 	while (gtk_tree_model_iter_next (GTK_TREE_MODEL(dialog->files_liststore),&iter));
 
+	/* let the user see the progress of 100% */
+	gtk_main_iteration();
+	g_usleep(G_USEC_PER_SEC / 4);
+
 	if (strlen(output->str)> 0)
 		xa_show_message_dialog (GTK_WINDOW (xa_main_window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Some errors occurred:"),output->str );
 
