@@ -17,6 +17,17 @@
  *  MA 02110-1301 USA.
  */
 
+#ifdef __APPLE__
+#include <libkern/OSByteOrder.h>
+#define le32toh(x) OSSwapLittleToHostInt32(x)
+#define htole32(x) OSSwapHostToLittleInt32(x)
+#elif defined(__FreeBSD__)
+#include <sys/endian.h>
+#else
+#include <endian.h>
+#endif
+#include <stdint.h>
+#include <stdio.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
@@ -110,7 +121,7 @@ gchar *xa_gzip_et_al_get_command (const gchar *program, gchar *workfile, gchar *
 	password_str = xa_gzip_et_al_password_str(password, type);
 	workfile = xa_escape_bad_chars(workfile, "\"");
 	archive = xa_quote_shell_command(archive, TRUE);
-	command = g_strconcat("sh -c \"", program, " ", workfile, password_str, type == XARCHIVETYPE_LRZIP ? " -fo " : " -c > ", archive, "\"", NULL);
+	command = g_strconcat("sh -c \"exec ", program, " ", workfile, password_str, type == XARCHIVETYPE_LRZIP ? " -fo " : " -c > ", archive, "\"", NULL);
 	g_free(archive);
 	g_free(workfile);
 	g_free(password_str);
@@ -427,12 +438,15 @@ void xa_gzip_et_al_list (XArchive *archive)
 {
 	GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_POINTER, G_TYPE_POINTER};
 	const gchar *titles[] = {_("Original Size"), _("Compressed"), _("Saving"), _("Date"), _("Time"), NULL, NULL};
-	const gchar *decompfile = "xa-tmp.decompressed";
-	gchar *password_str, *archive_path, *command, *workfile, buffer[12];
-	FILE *file;
+	const gchar *decompfile = "xa-tmp.decompressed", *framefile = "xa-tmp.framed";
+	gchar *password_str, *archive_path, *command, *workfile, buffer[4096];
+	FILE *file, *wfile;
 	struct stat st;
 	guint i;
 
+	if (!xa_create_working_directory(archive))
+		return;
+
 	if (archive->type == XARCHIVETYPE_LRZIP)
 	{
 		file = fopen(archive->path[0], "r");
@@ -458,18 +472,112 @@ void xa_gzip_et_al_list (XArchive *archive)
 		}
 	}
 
+	if (archive->type == XARCHIVETYPE_LZ4 && (archive->tag & 0xff) == 'm')
+	{
+		int skip, offset;
+		uint32_t uncompressed, blocksize;
+		guint8 flg, bd, hc, endmark[4] = "\x00\x00\x00\x00";
+		size_t bytes;
+
+		file = fopen(archive->path[0], "r");
+
+		if (file)
+		{
+			workfile = g_strconcat(archive->working_dir, "/", framefile, NULL);
+			wfile = fopen(workfile, "w");
+
+			if (wfile)
+			{
+				skip = (archive->tag == 'm' ? 8 : 16);   // magic
+
+				/* original (uncompressed) data size */
+				fseek(file, skip, SEEK_SET);
+				fread(&uncompressed, sizeof(uncompressed), 1, file);
+
+				/* create a lz4 frame header */
+
+				fwrite(LZ4_MAGIC, 4, 1, wfile);
+
+				flg = (0 << 7) + (1 << 6)   // version number
+				               + (1 << 5)   // block independence flag
+				               + (0 << 4)   // block checksum flag
+				               + (0 << 3)   // content size flag
+				               + (0 << 2)   // content checksum flag
+				               + (0 << 1)   // reserved
+				               + (0 << 0);  // dictionary id flag
+				fwrite(&flg, 1, 1, wfile);
+
+				if (le32toh(uncompressed) < 64 * 1024)          // 64 KB
+				{
+					bd = 0x40;
+					hc = 0x82;
+				}
+				else if (le32toh(uncompressed) < 256 * 1024)    // 256 KB
+				{
+					bd = 0x50;
+					hc = 0xfb;
+				}
+				else if (le32toh(uncompressed) < 1024 * 1024)   // 1 MB
+				{
+					bd = 0x60;
+					hc = 0x51;
+				}
+				else                                            // 4 MB
+				{
+					bd = 0x70;
+					hc = 0x73;
+				}
+
+				fwrite(&bd, 1, 1, wfile);   // block maximum size
+				fwrite(&hc, 1, 1, wfile);   // header checksum
+
+				offset = skip + 4;
+
+				fseek(file, 0, SEEK_END);
+				blocksize = htole32((uint32_t) (ftell(file) - offset));
+				fwrite(&blocksize, sizeof(blocksize), 1, wfile);
+
+				fseek(file, offset, SEEK_SET);
+
+				/* copy lz4 data */
+				while ((bytes = fread(buffer, 1, sizeof(buffer), file)) > 0)
+					fwrite(buffer, 1, bytes, wfile);
+
+				fwrite(endmark, sizeof(endmark), 1, wfile);
+
+				fclose(wfile);
+			}
+			else
+			{
+				fclose(file);
+				g_free(workfile);
+				return;
+			}
+
+			fclose(file);
+		}
+		else
+			return;
+
+		archive->path[2] = g_strdup(workfile);
+		archive->path[3] = xa_escape_bad_chars(workfile, ESCAPES);
+		g_free(workfile);
+	}
+	else
+	{
+		archive->path[2] = g_strdup(archive->path[0]);
+		archive->path[3] = g_strdup(archive->path[1]);
+	}
+
 	if (archive->has_password)
 		if (!xa_check_password(archive))
 			return;
 
-	if (!xa_create_working_directory(archive))
-		return;
-
 	password_str = xa_gzip_et_al_password_str(archive->password, archive->type);
-	archive_path = xa_quote_shell_command(archive->path[0], TRUE);
+	archive_path = xa_quote_shell_command(archive->path[2], TRUE);
 
 	archive->child_dir = g_strdup(archive->working_dir);
-	command = g_strconcat("sh -c \"", archiver[archive->type].program[0], " -d", password_str, " ", archive_path, lrzip ? " -fo " : " -c > ", decompfile, "\"", NULL);
+	command = g_strconcat("sh -c \"exec ", archiver[archive->type].program[0], " -d", password_str, " ", archive_path, lrzip ? " -fo " : " -c > ", decompfile, "\"", NULL);
 	xa_run_command(archive, command);
 	g_free(command);
 
@@ -503,6 +611,7 @@ void xa_gzip_et_al_list (XArchive *archive)
 			xa_gzip_et_al_can(archive, FALSE);
 
 			archive->archiver = &archiver[XARCHIVETYPE_TAR];
+			archive->status = XARCHIVESTATUS_LIST;
 
 			(*archive->archiver->ask)(archive);
 			(*archive->archiver->list)(archive);
@@ -597,7 +706,7 @@ void xa_gzip_et_al_list (XArchive *archive)
 				archive->columns = 10;
 			}
 
-			command = g_strconcat(archiver[archive->type].program[0], lrzip ? " -i" : " -l", xz ? " --robot " : (zstd ? "v " : " "), archive->path[1], NULL);
+			command = g_strconcat(archiver[archive->type].program[0], lrzip ? " -i" : " -l", xz ? " --robot " : (zstd ? "v " : " "), archive->path[3], NULL);
 
 			if (archive->type == XARCHIVETYPE_LRZIP)
 				archive->parse_output = xa_gzip_et_al_parse_lrzip;
@@ -661,7 +770,7 @@ void xa_gzip_et_al_test (XArchive *archive)
 	gchar *password_str, *command;
 
 	password_str = xa_gzip_et_al_password_str(archive->password, archive->type);
-	command = g_strconcat(archiver[archive->type].program[0], " -t", password_str, lrzip || lz4 ? " " : "v ", archive->path[1], NULL);
+	command = g_strconcat(archiver[archive->type].program[0], " -t", password_str, lrzip || lz4 ? " " : "v ", archive->path[3], NULL);
 	g_free(password_str);
 
 	xa_run_command(archive, command);
@@ -697,13 +806,13 @@ gboolean xa_gzip_et_al_extract (XArchive *archive, GSList *file_list)
 	out_dir = g_shell_unquote(archive->extraction_dir, NULL);
 	out_file = g_strconcat(out_dir, "/", filename, NULL);
 
-	archive_path = xa_quote_shell_command(archive->path[0], TRUE);
+	archive_path = xa_quote_shell_command(archive->path[2], TRUE);
 	extraction_dir = xa_quote_shell_command(archive->extraction_dir, FALSE);
 
 	if (archive->do_overwrite || !g_file_test(out_file, G_FILE_TEST_EXISTS))
 	{
 		password_str = xa_gzip_et_al_password_str(archive->password, archive->type);
-		command = g_strconcat("sh -c \"", archiver[archive->type].program[0], " -d", password_str, " ", archive_path, lrzip ? " -fo " : " -c > ", extraction_dir, "/", files_str, "\"", NULL);
+		command = g_strconcat("sh -c \"exec ", archiver[archive->type].program[0], " -d", password_str, " ", archive_path, lrzip ? " -fo " : " -c > ", extraction_dir, "/", files_str, "\"", NULL);
 		g_free(password_str);
 	}
 	else
@@ -770,7 +879,7 @@ void xa_gzip_et_al_add (XArchive *archive, GSList *file_list, gchar *compression
 
 	files = xa_quote_filenames(file_list, NULL, TRUE);
 	files_str = xa_escape_bad_chars(files->str, "\"");
-	archive_path = xa_quote_shell_command(archive->path[0], TRUE);
+	archive_path = xa_quote_shell_command(archive->path[2] ? archive->path[2] : archive->path[0], TRUE);
 
 	if (archive->do_move)
 		move = g_strconcat(" && rm", files_str, NULL);
@@ -778,7 +887,7 @@ void xa_gzip_et_al_add (XArchive *archive, GSList *file_list, gchar *compression
 		move = g_strdup("");
 
 	password_str = xa_gzip_et_al_password_str(archive->password, archive->type);
-	command = g_strconcat("sh -c \"", archiver[archive->type].program[0], " -", compress ? "b " : (lrzip ? "L " : ""), compression, password_str, files_str, lrzip ? " -fo " : " -c > ", archive_path, move, "\"", NULL);
+	command = g_strconcat("sh -c \"exec ", archiver[archive->type].program[0], " -", compress ? "b " : (lrzip ? "L " : ""), compression, password_str, files_str, lrzip ? " -fo " : " -c > ", archive_path, move, "\"", NULL);
 	g_free(password_str);
 
 	g_free(move);
