@@ -26,6 +26,7 @@
 #include "string_utils.h"
 #include "support.h"
 #include "window.h"
+#include "zip.h"
 
 GtkWidget *archive_dir_treeview;
 GtkWidget *archiver_data;
@@ -587,6 +588,34 @@ static void set_label (GtkWidget *label, gchar *text)
     g_free (tmp_markup);
 }
 
+static gchar *xa_progress_bar_message (XArchive *archive)
+{
+	gchar *message;
+
+	switch (archive->status)
+	{
+		case XARCHIVESTATUS_LIST:
+		  message = _("Opening archive:");
+			break;
+
+		case XARCHIVESTATUS_EXTRACT:
+		  message = _("Extracting from archive:");
+		  break;
+
+		case XARCHIVESTATUS_ADD:
+		  message = _("Adding to archive:");
+		  break;
+
+		default:
+		  message = "";
+		  break;
+	}
+
+	message = g_markup_printf_escaped("<b>%s</b>", message);
+
+	return message;
+}
+
 static void xa_cancel_progress_bar (GtkButton *button, GPid *pid)
 {
 	if (pid != NULL && *pid != 0)
@@ -616,7 +645,6 @@ static void xa_icon_theme_changed (GtkIconTheme *icon_theme, gpointer data)
 void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, gboolean show_sidebar, gboolean show_toolbar)
 {
 	GTK_COMPAT_TOOLTIPS;
-	GdkPixbuf *icon;
 	GtkIconSize tmp_toolbar_icon_size;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *archive_dir_renderer;
@@ -634,8 +662,7 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	xa_set_window_title (xa_main_window,NULL);
 
 	//g_signal_connect (G_OBJECT (icon_theme),"changed",G_CALLBACK (xa_icon_theme_changed),NULL);
-	icon = gtk_icon_theme_load_icon(icon_theme, "xarchiver", 24, (GtkIconLookupFlags) 0, NULL);
-	gtk_window_set_icon (GTK_WINDOW(xa_main_window),icon);
+	xa_set_xarchiver_icon(GTK_WINDOW(xa_main_window));
 	g_signal_connect (G_OBJECT (xa_main_window),"delete-event",G_CALLBACK (xa_quit_application),NULL);
 
 	vbox1 = gtk_vbox_new (FALSE,2);
@@ -1146,6 +1173,8 @@ gboolean xa_flash_led_indicator (XArchive *archive)
 	{
 		gtk_widget_show(green_led);
 		gtk_widget_hide(red_led);
+		archive->timeout = 0;
+
 		return FALSE;
 	}
 	if (gtk_widget_get_visible(green_led))
@@ -1244,7 +1273,7 @@ gboolean xa_check_password (XArchive *archive)
 	if (archive->password)
 		return TRUE;
 
-  	password_dialog = gtk_dialog_new_with_buttons(PACKAGE_NAME " " VERSION,
+  	password_dialog = gtk_dialog_new_with_buttons(PACKAGE_NAME,
 									GTK_WINDOW (xa_main_window),GTK_DIALOG_DESTROY_WITH_PARENT,
 									GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,GTK_STOCK_OK,GTK_RESPONSE_OK,NULL);
 
@@ -1253,6 +1282,8 @@ gboolean xa_check_password (XArchive *archive)
   	gtk_window_set_resizable (GTK_WINDOW (password_dialog),FALSE);
   	gtk_window_set_type_hint (GTK_WINDOW (password_dialog),GDK_WINDOW_TYPE_HINT_DIALOG);
 
+  	xa_set_xarchiver_icon(GTK_WINDOW(password_dialog));
+
   	dialog_vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(password_dialog));
   	gtk_widget_show (dialog_vbox1);
 
@@ -1277,7 +1308,12 @@ gboolean xa_check_password (XArchive *archive)
 	if (xa_main_window)
 		label_pwd_required = gtk_label_new(_("<span weight='bold' size='larger'>Enter password for:</span>"));
 	else
+	{
+		if (progress)
+			gtk_widget_hide(progress->window);
+
 		label_pwd_required = gtk_label_new(_("<span weight='bold' size='larger'>Password required for:</span>"));
+	}
 
   	gtk_widget_show (label_pwd_required);
   	gtk_box_pack_start (GTK_BOX (vbox2),label_pwd_required,FALSE,FALSE,0);
@@ -1540,13 +1576,21 @@ void xa_disable_delete_buttons (gboolean value)
 
 void xa_show_progress_bar (XArchive *archive)
 {
-	GtkWidget *vbox1, *vbox2, *message, *hbox1, *icon_pixbuf, *total_label, *action_area, *cancel_button;
+	GtkWidget *vbox1, *vbox2, *hbox1, *icon_pixbuf, *total_label, *action_area, *cancel_button;
 	GdkPixbuf *pixbuf;
 	gchar *text = NULL,*markup;
 
 	if (progress)
 	{
+		if (archive)
+		{
+			markup = xa_progress_bar_message(archive);
+			gtk_label_set_markup(GTK_LABEL(progress->message), markup);
+			g_free(markup);
+		}
+
 		gtk_widget_show(progress->window);
+
 		return;
 	}
 
@@ -1559,9 +1603,7 @@ void xa_show_progress_bar (XArchive *archive)
 	gtk_container_set_border_width(GTK_CONTAINER(progress->window), 6);
 	gtk_window_set_transient_for(GTK_WINDOW(progress->window), GTK_WINDOW(xa_main_window));
 
-	pixbuf = gtk_icon_theme_load_icon(icon_theme, "xarchiver", 24, (GtkIconLookupFlags) 0, NULL);
-	gtk_window_set_icon(GTK_WINDOW(progress->window), pixbuf);
-	g_object_unref(pixbuf);
+	xa_set_xarchiver_icon(GTK_WINDOW(progress->window));
 
 	vbox1 = gtk_vbox_new (FALSE,12);
   	gtk_container_add(GTK_CONTAINER(progress->window), vbox1);
@@ -1583,17 +1625,12 @@ void xa_show_progress_bar (XArchive *archive)
 
 	if (archive)
 	{
-		if (archive->status == XARCHIVESTATUS_EXTRACT)
-			text = _("Extracting from archive:");
-		else
-			text = _("Adding to archive:");
-
-		message = gtk_label_new("");
-		markup = g_markup_printf_escaped("<b>%s</b>", text);
-		gtk_label_set_markup(GTK_LABEL(message),markup);
+		progress->message = gtk_label_new(NULL);
+		markup = xa_progress_bar_message(archive);
+		gtk_label_set_markup(GTK_LABEL(progress->message), markup);
 		g_free (markup);
-		gtk_box_pack_start (GTK_BOX (vbox2),message,FALSE,FALSE,0);
-		gtk_misc_set_alignment (GTK_MISC (message),0,0.5);
+		gtk_box_pack_start(GTK_BOX(vbox2), progress->message, FALSE, FALSE, 0);
+		gtk_misc_set_alignment(GTK_MISC(progress->message), 0, 0.5);
 	}
 
 	progress->label = gtk_label_new(NULL);
@@ -1640,22 +1677,28 @@ void xa_increase_progress_bar (Progress *progress, gchar *filename, double perce
 		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress->bar), message);
 		g_free(message);
 
-		basename = g_path_get_basename(filename);
-		markup = g_markup_printf_escaped("<b>%s</b>", xa_set_max_width_chars_ellipsize(basename, 50, PANGO_ELLIPSIZE_END));
-		g_free(basename);
-		gtk_label_set_markup(GTK_LABEL(progress->label), markup);
-		g_free (markup);
+		if (filename)
+		{
+			basename = g_path_get_basename(filename);
+			markup = g_markup_printf_escaped("<b>%s</b>", xa_set_max_width_chars_ellipsize(basename, 50, PANGO_ELLIPSIZE_END));
+			g_free(basename);
+			gtk_label_set_markup(GTK_LABEL(progress->label), markup);
+			g_free(markup);
+		}
 }
 
-gboolean xa_pulse_progress_bar (gpointer user_data)
+gboolean xa_pulse_progress_bar (XArchive *archive)
 {
-	if (progress && gtk_widget_get_visible(progress->window))
+	if (gtk_widget_get_visible(progress->window))
 	{
 		gtk_progress_bar_pulse(GTK_PROGRESS_BAR(progress->bar));
 		return TRUE;
 	}
 	else
+	{
+		archive->timeout = 0;
 		return FALSE;
+	}
 }
 
 void xa_combo_box_text_append_compressor_types (GtkComboBoxText *combo_box_text)
@@ -1672,7 +1715,9 @@ void xa_combo_box_text_append_compressor_types (GtkComboBoxText *combo_box_text)
 			while (list)
 			{
 				if (list->data)
-					sorted = g_slist_append(sorted, list->data);
+					/* only zip can create epub compliant with the standards */
+					if (strcmp(list->data, "epub") != 0 || archiver[i].ask == xa_zip_ask)
+						sorted = g_slist_append(sorted, list->data);
 
 				list = list->next;
 			}
