@@ -518,6 +518,11 @@ static void xa_check_available_archivers ()
 		archiver[type].program[0] = path;
 		archiver[type].type = g_slist_append(archiver[type].type, "lz4");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lz4");
+		/* Mozilla's proprietary LZ4 Frame Formats */
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('m'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, "mozlz4");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('m' + 0x100));
+		archiver[type].tags = g_slist_append(archiver[type].tags, "mozJSSClz4");
 
 		archiver[type].ask = xa_gzip_et_al_ask;
 		archiver[type].list = xa_gzip_et_al_list;
@@ -974,6 +979,22 @@ static void xa_check_available_archivers ()
 	g_free(unar);
 }
 
+static gchar *xa_make_full_path (char *filename)
+{
+	gchar *cur_dir, *path;
+
+	if (*filename != '/')
+	{
+		cur_dir = g_get_current_dir();
+		path = g_strconcat(cur_dir, "/", filename, NULL);
+		g_free(cur_dir);
+	}
+	else
+		path = g_strdup(filename);
+
+	return path;
+}
+
 static XArchive *xa_init_structure_from_cmd_line (char *filename)
 {
 	XArchive *archive;
@@ -999,14 +1020,7 @@ static XArchive *xa_init_structure_from_cmd_line (char *filename)
 		return NULL;
 	}
 
-	if (*filename != '/')
-	{
-		gchar *cur_dir = g_get_current_dir();
-		path = g_strconcat(cur_dir, "/", filename, NULL);
-		g_free(cur_dir);
-	}
-	else
-		path = g_strdup(filename);
+	path = xa_make_full_path(filename);
 
 	archive->path[0] = g_strdup(path);
 	archive->path[1] = xa_escape_bad_chars(path, ESCAPES);
@@ -1194,13 +1208,37 @@ int main (int argc, char **argv)
 		/* Switch -c */
 		else if (opt_compress)
 		{
+			if (*opt_compress == 0)
+			{
+				xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't perform this action:"), _("You haven't selected any files to add!"));
+				g_free(opt_compress);
+				goto leave;
+			}
+
 			archive = xa_new_archive_dialog(opt_compress, NULL);
 
 			if (archive == NULL)
+			{
+				g_free(opt_compress);
 				goto leave;
+			}
 
 			if (archive->path[0] != NULL)
 			{
+				gchar *fname;
+				gboolean is_dir;
+
+				fname = xa_make_full_path(opt_compress);
+				is_dir = g_file_test(fname, G_FILE_TEST_IS_DIR);
+				g_free(fname);
+
+				if (SINGLE_FILE_COMPRESSOR(archive) && (is_dir || (argc > 1)))
+				{
+					xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("An error occurred!"), _("The archiver doesn't support compression of directories or multiple files!"));
+					g_free(opt_compress);
+					goto leave;
+				}
+
 				xa_create_working_directory(archive);
 				current_dir = g_path_get_dirname(opt_compress);
 				chdir(current_dir);
@@ -1227,7 +1265,7 @@ int main (int argc, char **argv)
 				xa_cmd_line_error(argv[1], _("Can't add files to the archive:"));
 				goto leave;
 			}
-			if (!archive->can_add)
+			if (!archive->can_add || SINGLE_FILE_COMPRESSOR(archive))
 			{
 				xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't add files to the archive:"), argv[1]);
 				goto leave;
