@@ -8,12 +8,13 @@
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA 02110-1301 USA.
  */
 
 #ifndef XARCHIVER_MAIN_H
@@ -26,6 +27,7 @@
 #define COPYRIGHT_YEAR "2005-2014"
 #define COPYRIGHT_HOLDER "Giuseppe Torelli"
 #define MAINTAINER "Ingo Brückl"
+#define MAINTAINER_YEAR "2015"
 
 extern GtkWidget *xa_main_window;
 
