@@ -24,6 +24,7 @@
 #include "main.h"
 #include "mime.h"
 #include "pref_dialog.h"
+#include "string_utils.h"
 #include "support.h"
 #include "window.h"
 
@@ -367,7 +368,7 @@ void xa_create_open_with_dialog (const gchar *filename, gchar *filenames, gint n
 	gtk_window_set_type_hint (GTK_WINDOW (data->dialog1), GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_window_set_transient_for(GTK_WINDOW(data->dialog1),GTK_WINDOW(xa_main_window));
 	gtk_container_set_border_width (GTK_CONTAINER (data->dialog1),5);
-	gtk_widget_set_size_request(data->dialog1, 380, 460);
+	gtk_widget_set_size_request(data->dialog1, 380, 485);
 	dialog_vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(data->dialog1));
 
 	vbox1 = gtk_vbox_new (FALSE, 5);
@@ -388,7 +389,7 @@ void xa_create_open_with_dialog (const gchar *filename, gchar *filenames, gint n
 
 		open_text = gtk_label_new("");
 		gtk_box_pack_start (GTK_BOX (hbox1),open_text,FALSE,FALSE,10);
-		text = g_strdup_printf(_("Open <i>%s</i> with:"),filename);
+		text = g_strdup_printf(_("Open <i>%s</i> with:"), xa_set_max_width_chars_ellipsize(filename, 40, PANGO_ELLIPSIZE_END));
 		gtk_label_set_use_markup (GTK_LABEL (open_text),TRUE);
 		gtk_label_set_markup (GTK_LABEL (open_text),text);
 		g_free(text);
