@@ -8,12 +8,13 @@
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA 02110-1301 USA.
  */
 
 #include <string.h>
@@ -25,24 +26,31 @@
 #include "support.h"
 #include "window.h"
 
-static int rar_version;
+int rar_version;
+
 static gboolean header_line, data_line, fname_line, last_line;
 
-int xa_rar_check_version (gchar *path)
+void xa_rar_check_version (gchar *path)
 {
-	gchar *output;
-
-	rar_version = 4;  // default version
+	gchar *output, *id;
+	gchar version;
 
 	g_spawn_command_line_sync(path, &output, NULL, NULL, NULL);
 
-	if (g_ascii_strncasecmp("\nRAR 5", output, 6) == 0 ||
-	    g_ascii_strncasecmp("\nUNRAR 5", output, 8) == 0)
-		rar_version = 5;
+	id = strstr(output, "\nRAR ");
+
+	if (!id)
+		id = strstr(output, "\nUNRAR ");
+
+	if (id)
+	{
+		version = *(strchr(id, ' ') + 1);
+
+		if (version > '1' && version <= '9')
+			rar_version = version - '0';
+	}
 
 	g_free(output);
-
-	return rar_version;
 }
 
 void xa_rar_ask (XArchive *archive)
@@ -259,7 +267,7 @@ static void xa_rar5_parse_output (gchar *line, XArchive *archive)
 	gpointer item[7];
 	unsigned short int i = 0;
 	unsigned int linesize, n, a;
-	gboolean encrypted = FALSE, dir = FALSE, iso_date;
+	gboolean encrypted = FALSE, dir = FALSE;
 	static gchar *filename, *end;
 
 	if (last_line)
@@ -362,9 +370,9 @@ static void xa_rar5_parse_output (gchar *line, XArchive *archive)
 	a = n;
 	for(; n < linesize && line[n] != ' '; n++);
 	line[n] = '\0';
-	item[i] = line + a;
-	iso_date = (strlen(item[i]) == 10);
-	if (!iso_date) item[i] = date_DD_MM_YY(item[i]);
+	item[i] = line + a;          // date is YYYY-MM-DD since v5.30
+	if (strlen(item[i]) != 10)   // and was DD-MM-YY before
+		item[i] = date_DD_MM_YY(item[i]);
 	i++;
 	n++;
 
@@ -386,8 +394,8 @@ static void xa_rar5_parse_output (gchar *line, XArchive *archive)
 
 	/* FileName */
 	line[linesize - 1] = '\0';
-	filename = g_strdup(line + (iso_date ? 66 : 64));   // date is YYYY-MM-DD since v5.30
-	                                                    // and was DD-MM-YY before
+	filename = g_strdup(line + n + 2);
+
 	/* Strip trailing whitespace */
 	end = filename + strlen(filename) - 1;
 	while(end >= filename && *end == ' ') end--;
@@ -481,7 +489,7 @@ void xa_rar_list (XArchive *archive)
 	archive->files = 0;
 
 
-	if (rar_version == 5)
+	if (rar_version >= 5)
 	{
 		const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
 		const gchar *titles[] = {_("Original Size"), _("Compressed"), _("Occupancy"), _("Date"), _("Time"), _("Attributes"), _("Checksum")};
@@ -568,7 +576,7 @@ void xa_rar_add (XArchive *archive, GSList *file_list, gchar *compression)
 	if (archive->location_path != NULL)
 		archive->child_dir = g_strdup(archive->working_dir);
 
-	if (rar_version == 5)
+	if (rar_version >= 5)
 	{
 		if (archive->tag == 5)
 			version_switch = " -ma5";
