@@ -346,6 +346,7 @@ void xa_tar_add (XArchive *archive, GSList *file_list, gchar *compression)
 		g_free(command);
 
 		command = xa_gzip_et_al_get_command(archiver[xa_tar_get_compressor_type(archive)].program[0], archive->path[2], archive->path[0], archive->password, xa_tar_get_compressor_type(archive));
+		archive->status = XARCHIVESTATUS_ADD;
 	}
 
 	g_string_free(files, TRUE);
@@ -368,6 +369,7 @@ void xa_tar_delete (XArchive *archive, GSList *file_list)
 		g_free(command);
 
 		command = xa_gzip_et_al_get_command(archiver[xa_tar_get_compressor_type(archive)].program[0], archive->path[2], archive->path[0], archive->password, xa_tar_get_compressor_type(archive));
+		archive->status = XARCHIVESTATUS_DELETE;
 	}
 
 	g_string_free(files, TRUE);
