@@ -30,6 +30,7 @@
 #include "add_dialog.h"
 #include "exe.h"
 #include "extract_dialog.h"
+#include "gzip_et_al.h"
 #include "interface.h"
 #include "main.h"
 #include "mime.h"
@@ -653,7 +654,7 @@ static void xa_comment_window_insert_in_archive (GtkButton *button, gpointer buf
 		break;
 
 		case XARCHIVETYPE_ZIP:
-		command = g_strconcat ("sh -c \"zip ",archive[idx]->path[1]," -z <",tmp,"\"",NULL);
+		command = g_strconcat ("sh -c \"exec zip ",archive[idx]->path[1]," -z <",tmp,"\"",NULL);
 		break;
 
 		default:
@@ -779,11 +780,11 @@ static void xa_expand_containing_directory (XArchive *archive)
 	}
 }
 
-void xa_child_processed (XAChildProcess process, gboolean success, XArchive *archive)
+void xa_child_processed (XAChildProcess process, guint8 exitstatus, XArchive *archive)
 {
-	static gboolean okay[XA_CHILD_PROCS];
+	static guint8 status[XA_CHILD_PROCS];
 
-	okay[process] = success;
+	status[process] = exitstatus;
 
 	if (process == XA_CHILD_EXIT)
 	{
@@ -798,10 +799,12 @@ void xa_child_processed (XAChildProcess process, gboolean success, XArchive *arc
 
 	if (--archive->child_ref == 0)
 	{
+		if (status[XA_CHILD_EXIT] == archive->exitstatus_ok) status[XA_CHILD_EXIT] = 0;
+
 		if (archive->output)
 			archive->output = g_slist_reverse(archive->output);
 
-		if (okay[XA_CHILD_EXIT] && okay[XA_CHILD_STDOUT] && okay[XA_CHILD_STDERR])
+		if (status[XA_CHILD_EXIT] == 0 && status[XA_CHILD_STDOUT] == 0 && status[XA_CHILD_STDERR] == 0)
 		{
 			if (xa_main_window)
 			{
@@ -908,9 +911,13 @@ void xa_show_archive_output (GtkMenuItem *menuitem, XArchive *this_archive)
 	if (xa_main_window)
 		title = _("Archiver output");
 	else
-		title = PACKAGE_NAME " " VERSION;
+		title = PACKAGE_NAME;
+
+	if (progress)
+		gtk_widget_hide(progress->window);
 
 	dialog = gtk_dialog_new_with_buttons(title, GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, NULL, NULL);
+	xa_set_xarchiver_icon(GTK_WINDOW(dialog));
 
 	show_cmd = gtk_dialog_add_button(GTK_DIALOG(dialog), _("Show command"), GTK_RESPONSE_APPLY);
 	gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
@@ -1003,6 +1010,11 @@ int xa_show_message_dialog (GtkWindow *window,int mode,int type,int button,const
 	int response;
 
 	dialog = gtk_message_dialog_new (window,mode,type,button,"%s",message1);
+	xa_set_xarchiver_icon(GTK_WINDOW(dialog));
+
+	if (!window)
+		gtk_window_set_title(GTK_WINDOW(dialog), PACKAGE_NAME);
+
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),"%s",message2);
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -1727,7 +1739,7 @@ void xa_about (GtkMenuItem *menuitem,gpointer user_data)
 			"documenters",documenters,
 			"translator_credits",_("translator-credits"),
 			"logo_icon_name","xarchiver",
-			"website","https://wiki.github.com/ib/xarchiver",
+			"website","https://github.com/ib/xarchiver/wiki",
 			"license","Copyright \xC2\xA9 " COPYRIGHT_YEAR " " COPYRIGHT_HOLDER " - Colossus <colossus73@gmail.com>\n"
 			          "Copyright \xC2\xA9 " MAINTAINER_YEAR " " MAINTAINER "\n\n"
 					"This program is free software; you can redistribute it and/or modify it\n"
@@ -1752,7 +1764,7 @@ void xa_about (GtkMenuItem *menuitem,gpointer user_data)
 ArchiveType xa_detect_archive_type (const gchar *filename)
 {
 	FILE *file;
-	unsigned char magic[14];
+	unsigned char magic[16];
 	long bytes;
 	uint32_t *uint32_magic = (uint32_t *) magic;
 	unsigned short *short_magic = (unsigned short *) magic;
@@ -1823,9 +1835,18 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 		xa.type = XARCHIVETYPE_LHA;
 	else if (memcmp(magic, "LRZI", 4) == 0)
 		xa.type = XARCHIVETYPE_LRZIP;
-	else if (memcmp(magic, "\x04\x22\x4d\x18", 4) == 0 ||
-	         memcmp(magic, "\x02\x21\x4c\x18", 4) == 0)
+	else if (memcmp(magic, LZ4_MAGIC, 4) == 0 ||
+	         memcmp(magic, "\x02\x21\x4c\x18", 4) == 0 ||
+	         memcmp(magic, "mozLz40\x00", 8) == 0 ||
+	         memcmp(magic, "mozJSSCLz40v001\x00", 16) == 0)
+	{
 		xa.type = XARCHIVETYPE_LZ4;
+
+		if (magic[3] == 'L')
+			xa.tag = 'm';
+		else if (magic[3] == 'J')
+			xa.tag = 'm' + 0x100;
+	}
 	else if (memcmp(magic, "LZIP", 4) == 0)
 		xa.type = XARCHIVETYPE_LZIP;
 	else if (memcmp(magic, "\x5d\x00\x00\x80", 4) == 0)
@@ -3059,3 +3080,12 @@ GtkWidget *xa_main_window_find_image (gchar *filename, GtkIconSize size)
 
   return file_image;
 }
+
+void xa_set_xarchiver_icon (GtkWindow *window)
+{
+	GdkPixbuf *pixbuf;
+
+	pixbuf = gtk_icon_theme_load_icon(icon_theme, "xarchiver", 24, (GtkIconLookupFlags) 0, NULL);
+	gtk_window_set_icon(window, pixbuf);
+	g_object_unref(pixbuf);
+}
